@@ -22,10 +22,12 @@ TEST_PARAMS = {'n_cells': 10, 'n_states': 4, 'state_seq_length': 5,
                'trial_time': 3.5, 'dt': 0.001, 'max_rate': 50, 'n_trials': 15,
                'min_state_dur': 0.05, 'noise': 0.01, 'baseline_dur': 1}
 
+
 HMM_PARAMS = {'hmm_id': None, 'taste': None, 'channel': None,  'unit_type':
               'single', 'dt': 0.001, 'threshold': 1e-4, 'max_iter': 1000,
               'n_cells': None, 'n_trials': None, 'time_start': 0, 'time_end':
-              2000, 'n_repeats': 3, 'n_states': 3}
+              2000, 'n_repeats': 3, 'n_states': 3, 'fitted': False}
+
 
 FACTORIAL_LOOKUP = np.array([math.factorial(x) for x in range(20)])
 
@@ -401,9 +403,10 @@ def rebin_spike_array(spikes, dt, time, new_dt):
 
 
 @memory.cache
-def get_hmm_spike_data(rec_dir, unit_type, channel, time_start=None, time_end=None, dt=None):
+def get_hmm_spike_data(rec_dir, unit_type, channel, time_start=None,
+                       time_end=None, dt=None, trials=None):
     units = query_units(rec_dir, unit_type)
-    time, spike_array = h5io.get_spike_data(rec_dir, units, channel)
+    time, spike_array = h5io.get_spike_data(rec_dir, units, channel, trials=trials)
     spike_array = spike_array.astype(np.int32)
     time = time.astype(np.float64)
     curr_dt = np.unique(np.diff(time))[0] / 1000
@@ -510,9 +513,11 @@ def fit_hmm_mp(rec_dir, params, h5_file=None):
     threshold = params['threshold']
     unit_type = params['unit_type']
     channel = params['channel']
+    n_trials = params['n_trials']
     spikes, dt, time = get_hmm_spike_data(rec_dir, unit_type, channel,
                                           time_start=time_start,
-                                          time_end=time_end, dt = dt)
+                                          time_end=time_end, dt=dt,
+                                          trials=n_trials)
     hmm = PoissonHMM(n_states, hmm_id=hmm_id)
     hmm.fit(spikes, dt, max_iter=max_iter, convergence_thresh=threshold)
     print('%s: Done Fitting for hmm %s' % (os.getpid(), hmm_id))
@@ -526,7 +531,7 @@ def fit_hmm_mp(rec_dir, params, h5_file=None):
 
         os.mknod(lock_file)
         try:
-            old_hmm, old_params = load_hmm_from_hdf5(h5_file, hmm_id)
+            old_hmm, _, old_params = load_hmm_from_hdf5(h5_file, hmm_id)
             if old_hmm is None:
                 print('%s: No existing HMM %s. Writing ...' % (pid, hmm_id))
                 hmmIO.write_hmm_to_hdf5(h5_file, hmm, time, params)
@@ -551,9 +556,9 @@ def fit_hmm_mp(rec_dir, params, h5_file=None):
 def load_hmm_from_hdf5(h5_file, hmm_id):
     existing_hmm = hmmIO.read_hmm_from_hdf5(h5_file, hmm_id)
     if existing_hmm is None:
-        return None, None
+        return None, None, None
 
-    PI, A, B, time, best_paths, params = existing_hmm
+    PI, A, B, time, best_paths, params, cost_hist, ll_hist = existing_hmm
     hmm = PoissonHMM(params['n_states'], PI=PI, A=A, B=B, hmm_id=hmm_id,
                      iteration=params.pop('n_iterations'))
     hmm.BIC = params.pop('BIC')
@@ -562,7 +567,9 @@ def load_hmm_from_hdf5(h5_file, hmm_id):
     hmm.cost = params.pop('cost')
     hmm.best_sequences = best_paths
     hmm.max_log_prob = params.pop('max_log_prob')
-    return hmm, params
+    hmm.cost_hist = cost_hist
+    hmm.ll_hist = ll_hist
+    return hmm, time, params
 
 
 class PoissonHMM(object):
@@ -581,6 +588,8 @@ class PoissonHMM(object):
         self.set_params(PI, A, B, iteration, spikes, dt)
         self.converged = False
         self.fitted = False
+        self.cost_hist = []
+        self.ll_hist = []
 
     def set_params(self, PI=None, A=None, B=None, iteration=0, spikes=None, dt=None):
         self.initial_distribution = PI
@@ -592,7 +601,7 @@ class PoissonHMM(object):
             self.cost = None
             self.BIC = None
             self.max_log_prob = None
-            self.bset_sequences = None
+            self.best_sequences = None
         elif PI is not None and A is not None and B is not None:
             spikes = spikes.astype(np.int32)
             self._update_cost(spikes, dt)
@@ -690,6 +699,8 @@ class PoissonHMM(object):
         iteration = self.iteration
         max_log_prob = self.max_log_prob
 
+        self.cost_hist.append(cost)
+        self.ll_hist.append(max_log_prob)
         if self.history is None:
             self.history = {}
             self.history['A'] = [A]
@@ -738,22 +749,49 @@ class PoissonHMM(object):
         B = self.emission
         cost = self.cost
 
-        dPI = np.sqrt(np.sum(np.power(oldPI - PI, 2)))
-        dA = np.sqrt(np.sum(np.power(oldA - A, 2)))
-        dB = np.sqrt(np.sum(np.power(oldB - B, 2)))
-        dCost = cost-oldCost
-        print('dPI = %f,  dA = %f,  dB = %f, dCost = %f, cost = %f'
-              % (dPI, dA, dB, dCost, cost))
+        # Instead of Euclidean distance, check that the change of EVERY item is less than threshold
+
+        ##################################################
+        # Old Way
+        ##################################################
+        # dPI = np.sqrt(np.sum(np.power(oldPI - PI, 2)))
+        # dA = np.sqrt(np.sum(np.power(oldA - A, 2)))
+        # dB = np.sqrt(np.sum(np.power(oldB - B, 2)))
+        # dCost = cost-oldCost
+        # print('dPI = %f,  dA = %f,  dB = %f, dCost = %f, cost = %f'
+        #       % (dPI, dA, dB, dCost, cost))
 
         # TODO: determine if this is reasonable
         # dB takes waaaaay longer to converge than the rest, i'm going to
         # double the thresh just for that
-        dB = dB/2
+        # dB = dB/2
 
-        if not all([x < thresh for x in [dPI, dA, dB]]):
-            return False
-        else:
+        #if not all([x < thresh for x in [dPI, dA, dB]]):
+        #    return False
+        #else:
+        #    return True
+
+        ##################################################
+        # New Way
+        ##################################################
+        dPI = np.abs(oldPI - PI)
+        dA = np.abs(oldA - A)
+        dB = np.abs(oldB - B)
+        delta = np.concatenate((dPI.ravel(), dA.ravel(), dB.ravel()))
+        dPI = np.sqrt(np.sum(np.power(dPI,2)))
+        dA = np.sqrt(np.sum(np.power(dA,2)))
+        dB = np.sqrt(np.sum(np.power(dB,2)))
+        dCost = cost-oldCost
+        print('dPI = %f,  dA = %f,  dB = %f, dCost = %f, cost = %f'
+              % (dPI, dA, dB, dCost, cost))
+
+        if all(delta <= thresh):
             return True
+        else:
+            return False
+
+
+
 
     def _update_cost(self, spikes, dt):
         spikes = spikes.astype('int32')
@@ -909,11 +947,9 @@ class HmmHandler(object):
         dim = dat.dig_in_mapping.query('exclude==False')
         tastes = dim['name'].tolist()
         self._orig_params = params
-        if params is None:
-            # Load params and fitted models
-            self.load_params()
-        else:
-            self.init_params(params)
+        self.load_params()
+        if params is not None:
+            self.add_params(params)
 
         if not os.path.isdir(save_dir):
             os.makedirs(save_dir)
@@ -924,47 +960,17 @@ class HmmHandler(object):
 
         hmmIO.setup_hmm_hdf5(self.h5_file)
 
-    def init_params(self, params):
-        dat = self._dataset
-        dim = dat.dig_in_mapping.query('exclude == False')
-        tastes = dim['name'].tolist()
-        dim = dim.set_index('name')
-        if not hasattr(dat, 'dig_in_trials'):
-            dat.create_trial_list()
-
-        trials = dat.dig_in_trials
-        data_params = []
-        fit_params = []
-        hmm_ids = []
-        for i, X in enumerate(it.product(params,tastes)):
-            p = X[0].copy()
-            t = X[1]
-            hmm_ids.append(i)
-            p['hmm_id'] = i
-            p['taste'] = t
-            p['channel'] = dim.loc[t, 'channel']
-            unit_names = query_units(dat, p['unit_type'])
-            p['n_cells'] = len(unit_names)
-            p['n_trials'] = len(trials.query('name == @t'))
-
-            data_params.append(p)
-            for i in range(p['n_repeats']):
-                fit_params.append(p.copy())
-
-        self._data_params = data_params
-        self._fit_params = fit_params
-        self._HMMs_fitted = dict.fromkeys(hmm_ids, False)
-
     def load_params(self):
+        self._data_params = []
+        self._fit_params = []
         h5_file = self.h5_file
         if not os.path.isfile(h5_file):
-            raise ValueError('No params to load')
+            return
 
-        rec_dir = self._dataset.root_dir
-        params = []
-        fit_params = []
-        fitted_models = {}
         with tables.open_file(h5_file, 'r') as hf5:
+            if 'data_overview' not in hf5.root:
+                return
+
             table = hf5.root.data_overview
             col_names = table.colnames
             for row in table[:]:
@@ -980,14 +986,7 @@ class HmmHandler(object):
                 _ = p.pop('n_iterations')
                 _ = p.pop('converged')
                 _ = p.pop('max_log_prob')
-                fitted_models[p['hmm_id']] = p.pop('fitted')
-                params.append(p)
-                for i in range(p['n_repeats']):
-                    fit_params.append(p.copy())
-
-        self._data_params = params
-        self._fit_params = fit_params
-        self._HMMs_fitted = fitted_models
+                self.add_params(p)
 
     def get_parameter_overview(self):
         df = pd.DataFrame(self._data_params)
@@ -1002,11 +1001,16 @@ class HmmHandler(object):
 
         return df
 
-    def run(self, parallel=True):
+    def run(self, parallel=True, overwrite=False):
         h5_file = self.h5_file
         rec_dir = self._dataset.root_dir
-        fit_params = self._fit_params
-        HMMs_fitted = {}
+        if overwrite:
+            fit_params = self._fit_params
+        else:
+            fit_params = [x for x in self._fit_params if not x['fitted']]
+
+        if len(fit_params) == 0:
+            return
 
         print('Running fittings')
         if parallel:
@@ -1026,25 +1030,91 @@ class HmmHandler(object):
         print('HMMs written to hdf5:')
         for hmm_id, written in results:
             print('%s : %s' % (hmm_id, written))
-            if written:
-                self._HMMs_fitted[hmm_id] = True
 
         self.plot_saved_models()
+        self.load_params()
 
     def plot_saved_models(self):
         print('Plotting saved models')
         data = self.get_data_overview().set_index('hmm_id')
         rec_dir = self._dataset.root_dir
         for i, row in data.iterrows():
-            hmm, params = load_hmm_from_hdf5(self.h5_file, i)
+            hmm, _, params = load_hmm_from_hdf5(self.h5_file, i)
             spikes, dt, time = get_hmm_spike_data(rec_dir, params['unit_type'],
                                                   params['channel'],
                                                   time_start=params['time_start'],
                                                   time_end=params['time_end'],
-                                                  dt = params['dt'])
+                                                  dt=params['dt'],
+                                                  trials=params['n_trials'])
             plot_dir = os.path.join(self.plot_dir, 'hmm_%s' % i)
             if not os.path.isdir(plot_dir):
                 os.makedirs(plot_dir)
 
             print('Plotting HMM %s...' % i)
             hmmplt.plot_hmm_figures(hmm, spikes, dt, time, save_dir=plot_dir)
+
+    def add_params(self, params):
+        if isinstance(params, list):
+            for p in params:
+                self.add_params(p)
+
+            return
+        elif not isinstance(params, dict):
+            raise ValueError('Input must  be a dict or list or dicts')
+
+        # Fill in blanks with defaults
+        for k, v in HMM_PARAMS.items():
+            if k not in params:
+                params[k] = v
+                print('Parameter %s not provided. Using default value: %s'
+                      % (k, repr(v)))
+
+        # Grab existing parameters
+        data_params = self._data_params
+        fit_params = self._fit_params
+
+        # Get taste and trial info from dataset
+        dat = self._dataset
+        dim = dat.dig_in_mapping.query('exclude == False and spike_array == True')
+        if params['taste'] is None:
+            tastes = dim['name'].tolist()
+        else:
+            tastes = [params['taste']]
+
+        dim = dim.set_index('name')
+        if not hasattr(dat, 'dig_in_trials'):
+            dat.create_trial_list()
+
+        trials = dat.dig_in_trials
+        hmm_ids = [x['hmm_id'] for x in data_params]
+        for t in tastes:
+            p = params.copy()
+            if len(hmm_ids) == 0:
+                hid = 0
+            else:
+                hid = hmm_ids[-1] + 1
+
+            p['taste'] = t
+            # Skip if parameter is already in parameter set
+            if any([hmmIO.compare_hmm_params(p, dp) for dp in data_params]):
+                print('Parameter set already in data_params, '
+                      'to re-fit run with overwrite=True')
+                continue
+
+            hmm_ids.append(hid)
+            p['hmm_id'] = hid
+            p['channel'] = dim.loc[t, 'channel']
+            unit_names = query_units(dat, p['unit_type'])
+            p['n_cells'] = len(unit_names)
+            if p['n_trials'] is None:
+                p['n_trials'] = len(trials.query('name == @t'))
+
+            data_params.append(p)
+            for i in range(p['n_repeats']):
+                fit_params.append(p.copy())
+
+        self._data_params = data_params
+        self._fit_params = fit_params
+
+
+
